//注意，注意，""../"表示退回上一级目录
import { userMobileLogin } from '../../api/wxlogin.js'
import { getMobileBtsParamList } from '../../api/btsparam.js'
import sysconfig from '../../sysconfig/sysconfig.js'

const app = getApp()
Page({
    data: {
        bestFrame: 0,
        bestMultiFrame: 0,
        bleDataList: [],//蓝牙设备队列，包括mac和rssi项
        btsParamList: [],//基站参数列表
        btsDataList: [],//基站数据列表
        btsMinCoordX: 0,
        btsMaxCoordX: 0,
        btsMinCoordY: 0,
        btsMaxCoordY: 0,

        canvasCtx: null, //画布
        canvasWidth: 0, //画布宽度
        canvasHeight: 0, //画布高度
        canvasPixelRatio: 1,//画布像素比例

        coordDataList: [],//定位坐标数据队列(所有定位坐标)
    },
    /**
     * 生命周期函数--监听页面加载
     */
    onLoad(options) {
        if (!app.data.loginFlag) {
            const myTimer = setInterval(() => {
                if (app.data.loginFlag) {
                    clearInterval(myTimer) // 停止定时器
                    this.init()//初始化
                }
            }, 500); // 每500毫秒检查一次
        } else {
            this.init()//初始化
        }
    },
    /**
     * 生命周期函数--监听页面初次渲染完成
     */
    onReady() {

    },
    /**
     * 生命周期函数--监听页面显示
     */
    onShow() {

    },
    /**
     * 生命周期函数--监听页面隐藏
     */
    onHide() {

    },
    /**
     * 生命周期函数--监听页面卸载
     */
    onUnload() {
        //页面关闭时释放wasm模块内存
        if (app.data.worker != null) {
            app.data.worker.postMessage({
                message: 'MAIN_WORKER_FREE_MEMORY',
                data: ''
            })
        }
    },
    /**
     * 页面相关事件处理函数--监听用户下拉动作
     */
    onPullDownRefresh() {

    },
    /**
     * 页面上拉触底事件的处理函数
     */
    onReachBottom() {

    },
    /**
     * 用户点击右上角分享
     */
    onShareAppMessage() {
    },
    //初始化
    init() {
        //创建线程
        app.createWorker(this.workerMessageListenerHome)

        //创建录音设备
        app.createRecorder()

        //拉取基站参数(注意：需要在用户登录成功后才拉取参数)
        this.getBtsParamList()

        //启动三轴加速计
        app.startImu()
    },
    //线程监听服务程序
    workerMessageListenerHome(res) {
        switch (res.message) {
            case 'WORKER_MAIN_START_MONITOR_BLE_BTS':
                app.openBleAdapter()//打开蓝牙设备
                break
            case 'WORKER_MAIN_ORGANIZE_INSTALL_BTS_DATA':
                this.organizeBtsDataForWasm()//组织基站数据
                break
            case 'WORKER_MAIN_START_REC':
                app.startRecord()//开始录音
                break
            case 'WORKER_MAIN_CURRENT_LOCATION_COORD'://当前定位坐标
                this.saveLocateCoord(res.data)
                break
            case 'WORKER_MAIN_NOTICE_DOWNLOAD_AUDIO_FILE':
                this.downloadAudioFile()
                break
            case 'WORKER_MAIN_NOTICE_READ_AUDIO_FILE':
                this.readAudioFile()
                break
            case 'WORKER_MAIN_SAVE_AUDIO_DATA':
                app.saveAduioDataToBin(res.data)
                break
            default:
                break
        }
    },
    //用户登录，登录成功后拉取基站参数
    userLogin() {
        // 用户登录登录（发送登录凭证[res.code]到后台换取openId、sessionKey和unionId）
        wx.login({
            success: (res) => {
                var paramData = {};
                paramData.code = res.code; //登录凭证
                paramData.domain_name = sysconfig.domanName; //系统二级域名（用二级域名换取项目编号）

                userMobileLogin(paramData).then((resp) => {
                    //保存token
                    const tokenKey = sysconfig.user.tokenKey
                    wx.setStorageSync(tokenKey, resp.data.data.token)

                    //保存用户数据
                    const userInfoKey = sysconfig.user.userInfoKey
                    wx.setStorageSync(userInfoKey, resp.data.data)

                    //拉取基站参数(注意：需要在用户登录成功后才拉取参数)
                    this.getBtsParamList()
                }).catch((err) => {
                    console.log("用户登录失败", err)
                })
            }
        })
    },
    //拉取基站参数(注意：需要在用户登录成功后才拉取参数)
    getBtsParamList() {
        //拉取基站参数
        getMobileBtsParamList().then((resBtsPram) => {
            this.data.btsParamList = resBtsPram.data.data

            //组织基站数据队列
            this.data.btsDataList = this.generateBtsDataList(this.data.btsParamList)

            //初始化画布
            this.initCanvas()
        }).catch((err) => {
            console.log("查询基站参数失败", err)
        })
    },
    //初始化画布
    initCanvas() {
        //获得屏幕数据
        let self = this
        wx.getSystemInfo({
            success: (res) => {
                self.data.canvasPixelRatio = res.pixelRatio
                let myCanvasWidth = res.windowWidth - 5
                self.setData({
                    canvasWidth: myCanvasWidth,
                    canvasHeight: myCanvasWidth
                })
            }
        })

        //为画布选择一个容器
        wx.createSelectorQuery().select('#homeCanvas').fields({ node: true, size: true }).exec((res) => {
            let canvas = res[0].node
            if (canvas) {
                this.data.canvasCtx = canvas.getContext('2d')
                canvas.width = res[0].width * this.data.canvasPixelRatio
                canvas.height = res[0].height * this.data.canvasPixelRatio
                this.data.canvasCtx.scale(this.data.canvasPixelRatio, this.data.canvasPixelRatio)

                //画网格
                this.drawLocateTrack(0, 0, 0, 0)

                //画基站坐标
                this.calMaxMinCoordXY(this.data.btsDataList)
            }
        })
    },
    //为wasm模块组织基站数据
    organizeBtsDataForWasm() {
        let dataList = []//数据队列
        let oneItem = {}
        let dataListLen = this.data.btsDataList.length

        //按照wasm模块要求的数据格式组织数据
        dataList.push(dataListLen)
        for (let i = 0; i < dataListLen; i++) {
            oneItem = this.data.btsDataList[i]
            dataList.push(oneItem.dwBuildId)
            dataList.push(oneItem.dwLayerId)

            dataList.push(oneItem.dwBtsId)
            dataList.push(oneItem.dwFreqIndex)
            dataList.push(oneItem.dwSlot)

            dataList.push(oneItem.dwCoordX)
            dataList.push(oneItem.dwCoordY)
            dataList.push(oneItem.dwCoordZ)

            //一维定位数据
            dataList.push(oneItem.dwLctType)
            dataList.push(oneItem.dwTwoBtsNum)
            for (let i = 0; i < oneItem.dwTwoBtsNum; i++) {
                dataList.push(oneItem.adwTwoBtsList[i])
            }
            for (let i = oneItem.dwTwoBtsNum; i < 6; i++) {//补零占位
                dataList.push(0)
            }

            //蓝牙mac
            for (let j = 0; j < 12; j++) {
                dataList.push(oneItem.adwBluetoothMac[j])
            }

            //wifi mac
            for (let k = 0; k < 12; k++) {
                dataList.push(oneItem.adwWifiMac[k])
            }
        }

        //将基站数据发送给worker
        if (app.data.worker != null && dataListLen > 0) {
            app.data.worker.postMessage({
                message: 'MAIN_WORKER_INSTALL_BTS_DATA',
                data: dataList//安装基站数据
            })
        }
    },
    //生成基站数据队列
    generateBtsDataList(btsParamList) {
        let dataList = []
        ////组织基站数据
        let btsNum = btsParamList.length;//基站数量
        for (let i = 0; i < btsNum; i++) {
            let btsParam = btsParamList[i]
            let oneItem = this.generateOneBtsData(btsParam.bts_id, btsParam.carrier_id, btsParam.timeslot, btsParam.coord_x, btsParam.coord_y, btsParam.coord_z, btsParam.ble_name, btsParam.location_type, btsParam.two_bts_num, btsParam.two_bts_list)
            dataList.push(oneItem)
        }

        return dataList
    },
    //生成一个基站数据
    //基站ID，频率ID(0 - 8)，时隙ID(0 - 3)，X坐标(单位厘米)，Y坐标(单位厘米)，Z坐标(单位厘米)，蓝牙MAC地址，定位类型，1维定位基站数，1维定位基站ID队列
    generateOneBtsData(dwBtsId, dwFreqIndex, dwSlot, dwCoordX, dwCoordY, dwCoordZ, macAddressList, dwLctType, dwTwoBtsNum, adwTwoBtsList) {
        let installBts = {}
        let oneCharHex = ''
        let oneCharHexValue = 0

        installBts.dwBuildId = 1
        installBts.dwLayerId = 1

        installBts.dwBtsId = dwBtsId
        installBts.dwFreqIndex = dwFreqIndex - 1
        installBts.dwSlot = dwSlot - 1

        installBts.dwCoordX = dwCoordX
        installBts.dwCoordY = dwCoordY
        installBts.dwCoordZ = dwCoordZ

        //定位类型，1：一维定位，2：二维定位，3：既支持一维定位也支持二维定位
        installBts.dwLctType = dwLctType

        //一维定位基站数量
        installBts.dwTwoBtsNum = dwTwoBtsNum;

        //一维定位基站ID队列
        installBts.adwTwoBtsList = []
        let btsIdList = adwTwoBtsList.split(",")
        let btsId = 0
        for (let k = 0; k < dwTwoBtsNum; k++) {
            btsId = parseInt(btsIdList[k])
            installBts.adwTwoBtsList.push(btsId)
        }

        //蓝牙mac
        installBts.adwBluetoothMac = []
        for (let i = 0; i < 12; i++) {
            oneCharHex = macAddressList[i]
            oneCharHexValue = app.hexCharToValue(oneCharHex)
            installBts.adwBluetoothMac.push(oneCharHexValue)
        }

        //wifi mac
        installBts.adwWifiMac = []
        for (let j = 0; j < 12; j++) {
            oneCharHex = macAddressList[j]
            oneCharHexValue = app.hexCharToValue(oneCharHex)
            installBts.adwWifiMac.push(oneCharHexValue)
        }

        return installBts
    },
    //计算基站围成的多边形最大最小坐标
    calMaxMinCoordXY(btsDataList) {
        let maxTmpX = -10000000
        let minTmpX = 10000000
        let maxTmpY = -10000000
        let minTmpY = 10000000
        let xTmp = 0
        let yTmp = 0

        let btsNum = btsDataList.length

        //计算基站围成的多边形坐标最大最小值
        for (let i = 0; i < btsNum; i++) {
            xTmp = btsDataList[i].dwCoordX
            if (xTmp > maxTmpX) {
                maxTmpX = xTmp
            }
            if (xTmp < minTmpX) {
                minTmpX = xTmp
            }

            yTmp = btsDataList[i].dwCoordY
            if (yTmp > maxTmpY) {
                maxTmpY = yTmp
            }
            if (yTmp < minTmpY) {
                minTmpY = yTmp
            }
        }

        //保存值
        this.setData({
            btsMinCoordX: minTmpX,
            btsMaxCoordX: maxTmpX,
            btsMinCoordY: minTmpY,
            btsMaxCoordY: maxTmpY
        })

        //画基站坐标
        this.drawLocateTrack(1, 0, 0, this.data.coordDataList)
    },
    //保存定位坐标
    saveLocateCoord(coord) {
        //保存定位坐标
        this.data.coordDataList.push(coord)

        //画定位轨迹
        this.drawLocateTrack(2, coord.x, coord.y, this.data.coordDataList)
    },
    //画定位轨迹，包括网格线、基站信息、定位轨迹和当前位置
    drawLocateTrack(type, coordinateX, coordinateY, coordDataList) {
        let i = 0
        let pos = 0
        let space = 10
        let lineCount = 40

        let ctxMinX = this.data.btsMinCoordX - 500
        let ctxMaxX = this.data.btsMaxCoordX + 500
        let ctxMinY = this.data.btsMinCoordY - 500
        let ctxMaxY = this.data.btsMaxCoordY + 500
        let ctxWidth = ctxMaxX - ctxMinX
        let ctxHeight = ctxMaxY - ctxMinY

        let radio = this.data.canvasWidth / ctxWidth
        let radioHeight = this.data.canvasHeight / ctxHeight
        if (radioHeight < radio) {
            radio = radioHeight
        }

        //发布已经初始化
        if (this.data.canvasCtx === null) {
            return
        }
        let positionCtx = this.data.canvasCtx
        positionCtx.fillStyle = "#0066FF"
        positionCtx.clearRect(0, 0, this.data.canvasWidth, this.data.canvasHeight)
        positionCtx.lineWidth = 0.5

        //// 画网格
        positionCtx.beginPath()
        positionCtx.strokeStyle = '#CCCCCC'
        //1、画网格横线
        space = parseFloat(this.data.canvasWidth) / lineCount
        for (i = 0; i < lineCount; i++) {
            pos = i * space
            positionCtx.moveTo(0, pos)
            positionCtx.lineTo(this.data.canvasWidth, pos)
        }
        //2、画网格竖线
        space = parseFloat(this.data.canvasHeight) / lineCount
        for (i = 0; i < lineCount; i++) {
            pos = i * space
            positionCtx.moveTo(pos, 0)
            positionCtx.lineTo(pos, this.data.canvasHeight)
        }
        positionCtx.stroke()
        if (type === 0) { //仅画网格
            return
        }

        //// 画基站
        positionCtx.beginPath()
        positionCtx.fillStyle = "#FF0000"
        positionCtx.strokeStyle = '#FF0000'
        for (i = 0; i < this.data.btsDataList.length; i++) {
            let x = (this.data.btsDataList[i].dwCoordX - ctxMinX) * radio
            let y = this.data.canvasHeight - (this.data.btsDataList[i].dwCoordY - ctxMinY) * radio
            const btsId = this.data.btsDataList[i].dwBtsId
            const freqIndex = this.data.btsDataList[i].dwFreqIndex
            const slot = this.data.btsDataList[i].dwSlot
            const coord = "[" + this.data.btsDataList[i].dwCoordX / 100 + "," + this.data.btsDataList[i].dwCoordY / 100 + "," + this.data.btsDataList[i].dwCoordZ / 100 + "]"

            positionCtx.fillStyle = "#FF0000"
            positionCtx.fillRect(x - 5, y - 5, 6, 6);//基站标识符

            positionCtx.fillStyle = "#000000"
            positionCtx.fillText(btsId, x + 5, y - 15)//画基站ID
            positionCtx.fillText("f: " + freqIndex, x + 5, y - 5)//画频率
            positionCtx.fillText("t: " + slot, x + 5, y + 5)//时隙
            positionCtx.fillText(coord, x + 5, y + 15)//坐标

        }
        if (type === 1) {//仅画基站
            return
        }

        //// 画轨迹线
        positionCtx.lineWidth = 1.2
        positionCtx.beginPath()
        positionCtx.strokeStyle = '#0A0A0A'
        for (i = 2; i < coordDataList.length; i++) {
            let point = coordDataList[i - 1]
            let x = (point.x - ctxMinX) * radio
            let y = this.data.canvasHeight - (point.y - ctxMinY) * radio
            positionCtx.moveTo(x, y)

            point = coordDataList[i]
            x = (point.x - ctxMinX) * radio
            y = this.data.canvasHeight - (point.y - ctxMinY) * radio
            positionCtx.lineTo(x, y)
        }
        positionCtx.stroke()

        //// 画当前位置
        positionCtx.beginPath()
        positionCtx.fillStyle = "#0000FF"
        positionCtx.strokeStyle = '#0000FF'
        let x = (coordinateX - ctxMinX) * radio
        let y = this.data.canvasHeight - (coordinateY - ctxMinY) * radio
        positionCtx.arc(x, y, 3, 0, 2 * Math.PI);
        positionCtx.fill()
    },
    //////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////下面是测试接口，已舍弃////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////
    //生成蓝牙基站数据
    generateBleBtsData() {
        let mac = "123456789ABC"
        let rssi = -58
        let item = {}
        item['mac'] = mac;
        item['rssi'] = rssi

        let dataList = []
        dataList.push(item)

        app.data.worker.postMessage({
            message: 'MAIN_WORKER_BLE_BTS_DATA',
            code: 100,
            data: dataList
        })
    },
    //从服务器上下载文件
    downloadAudioFile() {
        const url = "https://www.stp.intourism.cn/tdsadmin/line390-zt.wav"

        wx.downloadFile({
            url: url,
            success: (res) => {
                if (res.statusCode == 200) {
                    app.data.wavFilePath = res.tempFilePath
                    this.readAudioFile();
                }
            }
        })
    },
    // 读取音频文件
    readAudioFile() {
        let filePath = app.data.wavFilePath

        if (app.data.readFileTimes == null) {
            return
        }

        //90秒数据
        if (app.data.readFileTimes >= 360) {
            app.data.readFileTimes = 0
            return
        }

        const times = app.data.readFileTimes
        const fs = wx.getFileSystemManager();
        fs.readFile({
            filePath: `${filePath}`,
            position: 44 + times * 12000 * 2,
            length: 12000 * 2,//12000点，每点2个字节
            success: (res) => {
                app.data.readFileTimes++
                app.data.worker.postMessage({
                    message: 'MAIN_WORKER_AUDIO_DATA',
                    data: res.data
                })
            },
            fail(res) {
                console.log(res);
            }
        })
    },
})
